package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * A converter for converting video files
 */
public class VideoConverter extends AbstractConverter {

    /**
     * Instantiates a new video converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     * @param newExtension      <p>The extension of the new file.</p>
     */
    public VideoConverter(String ffprobePath, String ffmpegPath, String newExtension) {
        super(newExtension);
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
    }

    @Override
    public String[] builderCommand(String executable, File file, List<StreamObject> streams, String outFile) {
        List<String> command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, file.getName());

        if (this.DEBUG) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        List<AudioStream> audioStreams = filterStreamsByType(streams, "audio");
        List<VideoStream> videoStreams = filterStreamsByType(streams, "video");
        List<SubtitleStream> subtitleStreams = filterStreamsByType(streams, "subtitle");

        VideoStream videoStream = null;
        AudioStream audioStream = null;
        SubtitleStream subtitleStream = null;
        if (videoStreams.size() > 0) {
            videoStream = videoStreams.get(0);
        }
        if (audioStreams.size() > 0) {
            audioStream = audioStreams.get(0);
        }
        if (subtitleStreams.size() > 0) {
            subtitleStream = subtitleStreams.get(0);
        }

        addSubtitles(command, subtitleStream, videoStream, file);

        if (audioStreams.size() > 0) {
            command.add("-map");
            command.add("0:" + audioStream);
        }

        convertToStereo(command);

        command.add(outFile);
        return command.toArray(new String[0]);
    }

    /**
     * Converts the audio of a video to stereo
     *
     * @param command <p>The command list to add to.</p>
     */
    private void convertToStereo(List<String> command) {
        command.add("-af");
        command.add("pan=stereo|FL < 1.0*FL + 0.707*FC + 0.707*BL|FR < 1.0*FR + 0.707*FC + 0.707*BR");
    }

    @Override
    public String[] getValidFormats() {
        return videoFormats;
    }

    @Override
    public void convert(File file) throws IOException {
        processFile(file.getParentFile(), file);
    }
}